@@ -1,10 +1,11 @@
 use std::collections::HashMap;
 use std::io::{BufRead, BufReader, Write};
+use std::marker::PhantomData;
 use std::ops::{Deref, DerefMut};
 use std::process::{Child, ChildStdin, ChildStdout, Command, Stdio};
 use std::ptr::NonNull;
 use std::sync::mpsc::{self, Receiver, Sender};
-use std::sync::Arc;
+use std::sync::{Arc, Barrier};
 
 use nanoserde::{DeBin, SerBin};
 use parking_lot::{Mutex, MutexGuard, RwLock};
@@ -12,10 +13,13 @@ use parking_lot::{Mutex, MutexGuard, RwLock};
 use crate::{LiveMod, TrackedDataValue, Trigger};
 
 /// A handle to an external livemod viewer.
-#[derive(Clone)]
+///
+/// This handle is used to create [`ModVar`]s and track [`StaticModVar`]s. It must be kept alive
+/// for the user interface to continue running.
 pub struct LiveModHandle {
     sender: Sender<Message>,
     variables: Arc<RwLock<HashMap<String, ModVarHandle>>>,
+    barrier: Arc<Barrier>,
 }
 
 impl LiveModHandle {
@@ -44,6 +48,8 @@ impl LiveModHandle {
         let variables_arc2 = variables_arc1.clone();
         let variables_arc3 = variables_arc1.clone();
 
+        let barrier_arc1 = Arc::new(Barrier::new(2));
+        let barrier_arc2 = barrier_arc1.clone();
 
         std::thread::Builder::new()
             .name("livemod_input".to_owned())
@@ -55,7 +61,7 @@ impl LiveModHandle {
         std::thread::Builder::new()
             .name("livemod_output".to_owned())
             .spawn(|| {
-                output_thread(stdout, output_sender, variables_arc3);
+                output_thread(stdout, output_sender, variables_arc3, barrier_arc2);
                 drop(child_arc2);
             })
             .unwrap();
@@ -63,9 +69,11 @@ impl LiveModHandle {
         LiveModHandle {
             sender,
             variables: variables_arc1,
+            barrier: barrier_arc1,
         }
     }
 
+    /// Track an existing [`StaticModVar`]
     pub fn track_variable<T: LiveMod + 'static>(&self, name: &str, var: &'static StaticModVar<T>) {
         let var_handle = ModVarHandle {
             var: NonNull::from(&var.value),
@@ -76,14 +84,24 @@ impl LiveModHandle {
     }
 
     /// Create a variable and send it to the external viewer to be tracked.
-    pub fn create_variable<T: LiveMod + 'static>(&self, name: &str, var: T) -> ModVar<T> {
+    ///
+    /// The variable will be removed from the external viewer when it is dropped.
+    pub fn create_variable<'a, T: LiveMod + 'a>(&self, name: &str, var: T) -> ModVar<T> {
         let mod_var = ModVar {
             name: name.to_owned(),
-            handle: self.clone(),
             value: Box::new(Mutex::new(var)),
+            sender: self.sender.clone(),
+            variables: self.variables.clone(),
         };
         let var_handle = ModVarHandle {
-            var: NonNull::from(&*mod_var.value),
+            var: unsafe {
+                // SAFETY: The handle's value is stored on the heap, explicitly removed from the external viewer
+                // if it is dropped, but remaining valid if forgotten.
+                std::mem::transmute::<
+                    NonNull<Mutex<dyn LiveMod + 'a>>,
+                    NonNull<Mutex<dyn LiveMod + 'static>>,
+                >(NonNull::from(&*mod_var.value))
+            },
         };
         self.sender
             .send(Message::NewVariable(name.to_owned(), var_handle))
@@ -93,36 +111,48 @@ impl LiveModHandle {
     }
 }
 
+impl Drop for LiveModHandle {
+    fn drop(&mut self) {
+        self.sender.send(Message::Quit).unwrap();
+        self.barrier.wait();
+    }
+}
+
 /// A variable tracked by an external livemod viewer
 ///
 /// A `ModVar` cannot be created directly, and must be created using the [`LiveModHandle::create_variable`] method.
 pub struct ModVar<T> {
     name: String,
-    handle: LiveModHandle,
     value: Box<Mutex<T>>,
+    sender: Sender<Message>,
+    variables: Arc<RwLock<HashMap<String, ModVarHandle>>>,
 }
 
-impl<T: LiveMod + 'static> ModVar<T> {
+impl<T: LiveMod> ModVar<T> {
+    /// Get an immutable reference to the value in this `ModVar`. The value will not be changed
+    /// by the external viewer while this reference is held.
     pub fn lock(&self) -> ModVarGuard<T> {
         ModVarGuard(self.value.lock())
     }
 
+    /// Get a mutable reference to the value in thie `ModVar` The value will not be changed
+    /// by the external viewer while this reference is held. The value in the external viewer
+    /// will be updated if and only if the `ModVarMutGuard` is dereferenced mutably.
     pub fn lock_mut(&mut self) -> ModVarMutGuard<T> {
-        ModVarMutGuard(self.value.lock(), Some(UpdateMessage::new(&self)))
+        ModVarMutGuard(self.value.lock(), Some(UpdateMessage::new(self)))
     }
 }
 
 impl<T> Drop for ModVar<T> {
     fn drop(&mut self) {
-        self.handle
-            .sender
+        self.sender
             .send(Message::RemoveVariable(self.name.clone()))
             .unwrap();
-        self.handle.variables.write().remove(&self.name);
+        self.variables.write().remove(&self.name);
     }
 }
 
-/// A static trackable livemod variable
+/// A static trackable livemod variable.
 pub struct StaticModVar<T> {
     value: Mutex<T>,
 }
@@ -134,11 +164,14 @@ impl<T> StaticModVar<T> {
         }
     }
 
+    /// Get an immutable reference to the value in this `ModVar`. The value will not be changed
+    /// by the external viewer while this reference is held.
     pub fn lock(&self) -> ModVarGuard<T> {
         ModVarGuard(self.value.lock())
     }
 }
 
+/// An immutable lock of a [`ModVar`] or [`StaticModVar`]. Can be dereferenced to get the contained data.
 pub struct ModVarGuard<'a, T>(MutexGuard<'a, T>);
 
 impl<'a, T> Deref for ModVarGuard<'a, T> {
@@ -149,7 +182,10 @@ impl<'a, T> Deref for ModVarGuard<'a, T> {
     }
 }
 
-pub struct ModVarMutGuard<'a, T>(MutexGuard<'a, T>, Option<UpdateMessage>);
+/// A mutable lock of a [`ModVar`]. Can be dereferenced to get the contained data, and modified.
+///
+/// The value is updated in the external viewer if and only if this guard is dereferenced mutably.
+pub struct ModVarMutGuard<'a, T>(MutexGuard<'a, T>, Option<UpdateMessage<'a>>);
 
 impl<'a, T> Deref for ModVarMutGuard<'a, T> {
     type Target = T;
@@ -168,20 +204,28 @@ impl<'a, T> DerefMut for ModVarMutGuard<'a, T> {
     }
 }
 
-struct UpdateMessage {
+struct UpdateMessage<'a> {
     name: String,
     handle: ModVarHandle,
     sender: Sender<Message>,
+    _marker: PhantomData<&'a ModVarHandle>,
 }
 
-impl UpdateMessage {
-    fn new<T: LiveMod + 'static>(var: &ModVar<T>) -> UpdateMessage {
+impl UpdateMessage<'_> {
+    fn new<'a, T: LiveMod + 'a>(var: &'a ModVar<T>) -> UpdateMessage<'a> {
         UpdateMessage {
             name: var.name.clone(),
             handle: ModVarHandle {
-                var: NonNull::from(&*var.value),
+                var: unsafe {
+                    // SAFETY: The value lives as long as the ModVar which we are borrowing
+                    std::mem::transmute::<
+                        NonNull<Mutex<dyn LiveMod + 'a>>,
+                        NonNull<Mutex<dyn LiveMod + 'static>>,
+                    >(NonNull::from(&*var.value))
+                },
             },
-            sender: var.handle.sender.clone(),
+            sender: var.sender.clone(),
+            _marker: std::marker::PhantomData,
         }
     }
 
@@ -205,6 +249,7 @@ enum Message {
     UpdatedVariable(String, ModVarHandle),
     RemoveVariable(String),
     UpdatedRepr(Vec<String>),
+    Quit,
 }
 
 struct ChildDropper {
@@ -222,75 +267,67 @@ fn input_thread(
     recv: Receiver<Message>,
     variables: Arc<RwLock<HashMap<String, ModVarHandle>>>,
 ) {
-    loop {
-        match recv.recv() {
-            Ok(message) => match message {
-                Message::NewVariable(name, handle) => {
-                    let var = unsafe { handle.var.as_ref() }.lock();
-                    writeln!(
-                        input,
-                        "n{};{};{}",
-                        &name,
-                        base64::encode_config(
-                            var.repr_default().serialize_bin(),
-                            base64::STANDARD_NO_PAD
-                        ),
-                        base64::encode_config(
-                            var.get_self().serialize_bin(),
-                            base64::STANDARD_NO_PAD
-                        ),
-                    )
-                    .unwrap();
-                    variables.write().insert(name, handle);
-                }
-                Message::UpdatedVariable(name, handle) => {
-                    let var = unsafe { handle.var.as_ref() }.lock();
-                    writeln!(
-                        input,
-                        "s{};{}",
-                        &name,
-                        base64::encode_config(
-                            var.get_self().serialize_bin(),
-                            base64::STANDARD_NO_PAD
-                        ),
-                    )
-                    .unwrap();
-                }
-                Message::UpdatedRepr(path) => {
-                    // Get the 'base' variable from our HashMap
-                    let base = path.first().unwrap();
-                    let mut var_handle =
-                        unsafe { &mut *variables.read().get(base).unwrap().var.as_ref().lock() };
+    while let Ok(message) = recv.recv() {
+        match message {
+            Message::NewVariable(name, handle) => {
+                let var = unsafe { handle.var.as_ref() }.lock();
+                writeln!(
+                    input,
+                    "n{};{};{}",
+                    &name,
+                    base64::encode_config(
+                        var.repr_default().serialize_bin(),
+                        base64::STANDARD_NO_PAD
+                    ),
+                    base64::encode_config(var.get_self().serialize_bin(), base64::STANDARD_NO_PAD),
+                )
+                .unwrap();
+                variables.write().insert(name, handle);
+            }
+            Message::UpdatedVariable(name, handle) => {
+                let var = unsafe { handle.var.as_ref() }.lock();
+                writeln!(
+                    input,
+                    "s{};{}",
+                    &name,
+                    base64::encode_config(var.get_self().serialize_bin(), base64::STANDARD_NO_PAD),
+                )
+                .unwrap();
+            }
+            Message::UpdatedRepr(path) => {
+                // Get the 'base' variable from our HashMap
+                let base = path.first().unwrap();
+                let mut var_handle =
+                    unsafe { &mut *variables.read().get(base).unwrap().var.as_ref().lock() };
 
-                    // Follow the 'path' of field names, if needed
-                    if path.len() > 2 {
-                        for name in &path[1..=path.len() - 2] {
-                            var_handle = var_handle.get_named_value(name);
-                        }
+                // Follow the 'path' of field names, if needed
+                if path.len() > 2 {
+                    for name in &path[1..=path.len() - 2] {
+                        var_handle = var_handle.get_named_value(name);
                     }
+                }
 
-                    writeln!(
-                        input,
-                        "u{};{};{}",
-                        path.into_iter().reduce(|acc, v| format!("{}:{}", acc, v)).unwrap(),
-                        base64::encode_config(
-                            var_handle.repr_default().serialize_bin(),
-                            base64::STANDARD_NO_PAD
-                        ),
-                        base64::encode_config(
-                            var_handle.get_self().serialize_bin(),
-                            base64::STANDARD_NO_PAD
-                        ),                       
-                    )
-                    .unwrap();
-                }
-                Message::RemoveVariable(name) => {
-                    writeln!(input, "r{}", &name).unwrap();
-                }
-            },
-            Err(mpsc::RecvError) => {
-                // The LiveModHandle which spawned this thread has
-                // been destroyed, so quit and clean up now.
+                writeln!(
+                    input,
+                    "u{};{};{}",
+                    path.into_iter()
+                        .reduce(|acc, v| format!("{}:{}", acc, v))
+                        .unwrap(),
+                    base64::encode_config(
+                        var_handle.repr_default().serialize_bin(),
+                        base64::STANDARD_NO_PAD
+                    ),
+                    base64::encode_config(
+                        var_handle.get_self().serialize_bin(),
+                        base64::STANDARD_NO_PAD
+                    ),
+                )
+                .unwrap();
+            }
+            Message::RemoveVariable(name) => {
+                writeln!(input, "r{}", &name).unwrap();
+            }
+            Message::Quit => {
                 break;
             }
         }
@@ -303,6 +340,7 @@ fn output_thread(
     output: ChildStdout,
     sender: Sender<Message>,
     variables: Arc<RwLock<HashMap<String, ModVarHandle>>>,
+    barrier: Arc<Barrier>,
 ) {
     for line in BufReader::new(output).lines() {
         let line = line.as_ref().unwrap().as_bytes();
@@ -319,32 +357,52 @@ fn output_thread(
                     .split(|&b| b == b':' || b == b';')
                     .map(std::str::from_utf8)
                     .collect::<Result<Vec<_>, _>>()
-                    .unwrap();
+                    .expect("Invalid text encoding received from viewer");
 
                 // Get the 'base' variable from our HashMap
                 let base = namespaced_name.first().unwrap();
-                let mut var_handle =
-                    unsafe { &mut *variables.read().get(*base).unwrap().var.as_ref().lock() };
+                let mut referenced_var = unsafe {
+                    &mut *match variables.read().get(*base) {
+                        Some(base_handle) => base_handle,
+                        None => {
+                            // The variable has already been removed
+                            continue;
+                        }
+                    }
+                    .var
+                    // SAFETY: Pointers are valid as long as they are in the map
+                    .as_ref()
+                    .lock()
+                };
 
                 // Follow the 'path' of field names, if needed, ignoring the part after ';'
                 if namespaced_name.len() > 2 {
                     for name in &namespaced_name[1..=namespaced_name.len() - 2] {
-                        var_handle = var_handle.get_named_value(name);
+                        referenced_var = referenced_var.get_named_value(name);
                     }
                 }
 
                 // Set the variable
-                if var_handle.trigger(Trigger::Set(
+                if referenced_var.trigger(Trigger::Set(
                     TrackedDataValue::deserialize_bin(
                         &base64::decode_config(
                             namespaced_name.last().unwrap(),
                             base64::STANDARD_NO_PAD,
                         )
-                        .unwrap(),
+                        .expect("Invalid base64 data received from viewer"),
                     )
-                    .unwrap(),
+                    .expect("Invalid data received from viewer"),
                 )) {
-                    sender.send(Message::UpdatedRepr(namespaced_name.into_iter().map(str::to_owned).collect())).unwrap();
+                    let len = namespaced_name.len() - 1;
+                    sender
+                        .send(Message::UpdatedRepr(
+                            namespaced_name
+                                .into_iter()
+                                .take(len)
+                                .map(str::to_owned)
+                                .collect(),
+                        ))
+                        .unwrap();
                 }
             }
             b't' => {
@@ -353,26 +411,45 @@ fn output_thread(
                     .split(|&b| b == b':' || b == b';')
                     .map(std::str::from_utf8)
                     .collect::<Result<Vec<_>, _>>()
-                    .unwrap();
+                    .expect("Invalid text encoding received from viewer");
 
                 // Get the 'base' variable from our HashMap
                 let base = namespaced_name.first().unwrap();
-                let mut var_handle =
-                    unsafe { &mut *variables.read().get(*base).unwrap().var.as_ref().lock() };
+                let mut referenced_var = unsafe {
+                    &mut *match variables.read().get(*base) {
+                        Some(base_handle) => base_handle,
+                        None => {
+                            // The variable has already been removed
+                            continue;
+                        }
+                    }
+                    .var
+                    // SAFETY: Pointers are valid as long as they are in the map
+                    .as_ref()
+                    .lock()
+                };
 
                 // Follow the 'path' of field names, if needed, ignoring the part after ';'
                 if namespaced_name.len() > 2 {
                     for name in &namespaced_name[1..=namespaced_name.len() - 2] {
-                        var_handle = var_handle.get_named_value(name);
+                        referenced_var = referenced_var.get_named_value(name);
                     }
                 }
 
                 // Trigger the action denoted by the last element of the name
-                if var_handle.trigger(Trigger::Trigger(
+                if referenced_var.trigger(Trigger::Trigger(
                     (*namespaced_name.last().unwrap()).to_owned(),
                 )) {
                     let len = namespaced_name.len() - 1;
-                    sender.send(Message::UpdatedRepr(namespaced_name.into_iter().take(len).map(str::to_owned).collect())).unwrap();
+                    sender
+                        .send(Message::UpdatedRepr(
+                            namespaced_name
+                                .into_iter()
+                                .take(len)
+                                .map(str::to_owned)
+                                .collect(),
+                        ))
+                        .unwrap();
                 }
             }
             _ => {
@@ -380,4 +457,5 @@ fn output_thread(
             }
         }
     }
+    barrier.wait();
 }
